import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
from scipy.optimize import curve_fit

df = pd.read_csv("blender_distance_analysis.csv", header=0, index_col=0)
df = df.sort_values(by=["shape", "translation"])

shape_list = ["Cube", "Cone", "Cylinder", "Sphere", "Torus"]
color_list = ["#4477AA", "#EE6677", "#228833", "#CCBB44", "#66CCEE"]
linestyle_list = ["-", "--", "-.", ":", "-"]
marker_list = ["s", "^", "|", "o", "*"]
format_list = ["-s", "--^", "-.|", ":o", "-*"]

fig = plt.figure(figsize=(7.25,7.25))
ax = fig.add_subplot(111)

artist_list = []
for shape, c, fmt in zip(shape_list, color_list, format_list):
    selection = df["shape"] == shape

    ax.plot(((df[selection]["translation"]-20) * -1),
                df[selection]["volume_error"],
                fmt,
                color=c)


legend_handles = [mlines.Line2D([],
                                [],
                                color=c,
                                marker=m,
                                linestyle=l,
                                label=shape)
                  for c, m, l, shape in zip(color_list,
                                            marker_list,
                                            linestyle_list,
                                            shape_list)]
ax.legend(handles=legend_handles)

ax.set_xlabel("Distance from Object / m")
ax.set_ylabel("Volume % Error")
# ax.set_xticks(np.linspace(-100,100,5))
# ax.set_yticks(np.linspace(0,50,6))
plt.show()
plt.close()


fig = plt.figure(figsize=(7.25,7.25))
ax = fig.add_subplot(111)

artist_list = []
for shape, c, fmt in zip(shape_list, color_list, format_list):
    selection = df["shape"] == shape

    ax.plot((df[selection]["translation"]-20) * -1,
                df[selection]["height_error"],
                fmt,
                color=c)


legend_handles = [mlines.Line2D([],
                                [],
                                color=c,
                                marker=m,
                                linestyle=l,
                                label=shape)
                  for c, m, l, shape in zip(color_list,
                                            marker_list,
                                            linestyle_list,
                                            shape_list)]
ax.legend(handles=legend_handles)

ax.set_xlabel("Distance from Object / m")
ax.set_ylabel("Height % Error")
# ax.set_xticks(np.linspace(-100,100,5))
# ax.set_yticks(np.linspace(0,50,6))
plt.show()
plt.close()

def fit_func(x, a, b):
    return x*a + b

fig = plt.figure(figsize=(7.25,4.2))
ax = fig.add_subplot(111)

artist_list = []
for shape, c, fmt in zip(shape_list, color_list, marker_list):
    selection = df["shape"] == shape

    popt, pcov = curve_fit(fit_func, 1/((df[selection]["translation"]-20) * -1), df[selection]["volume_error"])

    ax.scatter(1/((df[selection]["translation"]-20) * -1),
                   df[selection]["volume_error"],
                   marker = fmt,
                   color=c)
    x = np.linspace(-0.001, 0.1, 100)
    y = fit_func(x, *popt)
    ax.plot(x, y, color=c)
    print(shape)
    print(*popt)


legend_handles = [mlines.Line2D([],
                                [],
                                color=c,
                                marker=m,
                                linestyle=l,
                                label=shape)
                  for c, m, l, shape in zip(color_list,
                                            marker_list,
                                            linestyle_list,
                                            shape_list)]
ax.legend(handles=legend_handles)

ax.set_xlabel(r"1 / $Z_c$ (m$^{-1}$)", fontsize=12)
ax.set_ylabel("Volume % Error", fontsize=12)
ax.set_xlim(0,0.105)
ax.set_ylim(0,150)
plt.rc('xtick', labelsize=10)
plt.rc('ytick', labelsize=10)
# ax.set_xticks(np.linspace(-100,100,5))
# ax.set_yticks(np.linspace(0,50,6))
plt.savefig("distance-analysis\\plots\\inverse_distance_plot.png", dpi=300)
plt.savefig("distance-analysis\\plots\\inverse_distance_plot.svg")
plt.savefig("distance-analysis\\plots\\inverse_distance_plot.eps")
plt.savefig("distance-analysis\\plots\\inverse_distance_plot.tif", dpi=1000)
plt.close()
