import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.lines as mlines

df = pd.read_csv("blender_sensitivity_analysis.csv", header=0, index_col=0)
df = df[df["scale"] == 3840]
df["true_scale"] = df["scale"] / 3840
df["Z_s"] = (df["translation"] / df["translation"].max()) * 100
df["a_s"] = (df["angle"] / df["angle"].max()) * 100
df["o_s"] = (df["offset"] / df["offset"].max()) * 100
df["S_s"] = ((df["true_scale"]) - 1 / df["true_scale"].max()) * 100

shape_list = ["Cube", "Cone", "Cylinder", "Sphere", "Torus"]
color_scheme = ["#4477AA", "#EE6677", "#228833", "#CCBB44", "#66CCEE"]
ylims = [(10, 40), (25, 55), (25, 55), (25, 55), (125, 155)]

plot_count = 1
for i, shape in enumerate(shape_list):
    shape_rule = df["shape"] == shape
    translation_rule = df["translation"] == 0
    angle_rule = df["angle"] == 0
    offset_rule = df["offset"] == 0

    plt.rc('xtick', labelsize=14)
    plt.rc('ytick', labelsize=14)
    fig = plt.figure(figsize=(8.0, 4.4))
    ax = fig.add_subplot(111)
    selection = np.logical_and(shape_rule,
                np.logical_and(angle_rule, offset_rule))
    ax.plot(df[selection].sort_values(by=["Z_s"])["Z_s"][::-1],
            df[selection].sort_values(by=["Z_s"])["volume_error"],
            "-o",
            color=color_scheme[0],
            lw=2.5, markersize=16)
    selection = np.logical_and(shape_rule,
                np.logical_and(translation_rule, offset_rule))
    ax.plot(df[selection].sort_values(by=["a_s"])["a_s"],
            df[selection].sort_values(by=["a_s"])["volume_error"],
            "--x",
            color=color_scheme[1],
            lw=2.5, markersize=16)
    selection = np.logical_and(shape_rule,
                np.logical_and(angle_rule, translation_rule))
    ax.plot(df[selection].sort_values(by=["o_s"])["o_s"],
            df[selection].sort_values(by=["o_s"])["volume_error"],
            "-.D",
            color=color_scheme[2],
            lw=2.5, markersize=16)
    # ax.plot(df[selection].sort_values(by=["S_s"])["S_s"],
    #         df[selection].sort_values(by=["S_s"])["volume_error"],
    #         ":*",
    #         color=color_scheme[3],
    #         lw=1.5)

    ax.set_xlabel("% Variation in Sensitivity Parameter", fontsize=18)
    ax.set_ylabel("Volume % Error", fontsize=18)
    ax.set_xticks(np.linspace(-100,100,5))
    ax.set_xticklabels(np.linspace(-100,100,5, dtype=np.int32), fontsize=12)
    #if plot_count < 3:
        #ax.set_xticklabels(np.repeat("", 5))
        #ax.set_xlabel("")
    # ax.set_yticks(np.linspace(0,50,6))
    # ax.set_yticklabels(np.linspace(0,50,6, dtype=np.int32), fontsize=8)
    #if plot_count % 2 == 0:
        #ax.set_yticklabels(np.repeat("", 5))
        #ax.set_ylabel("")
    ax.set_ylim(ylims[i])
    # ax.set_title(shape, fontsize=12)


    if plot_count == 1:
        translation_handle = mlines.Line2D([], [], color=color_scheme[0], ls="-", marker="o", label="Distance from object")
        angle_handle = mlines.Line2D([], [], color=color_scheme[1], ls="--", marker="x", label="Rotational offset")
        offset_handle = mlines.Line2D([], [], color=color_scheme[2], ls="-.", marker="D", label="Translational offset")
        scale_handle = mlines.Line2D([], [], color=color_scheme[3], ls=":", marker="*", label="Resolution")
        ax.legend(handles=[translation_handle, angle_handle, offset_handle], loc="upper right", fontsize=16)

    plt.tight_layout()
    plt.savefig("plots\\volume_sensitivity_{}.png".format(shape), dpi=300)
    plt.savefig("plots\\volume_sensitivity_{}.svg".format(shape))
    plt.close()

    plot_count += 1
