import sys
import os
sys.path.insert(0, os.path.join(os.getcwd(),"..\\"))
from blender_functions import *

def main():
    start = time()
    # Define varying parameters for the sensitivity study
    default_resolution = np.array([3840, 2160])
    shape_list = ["Cube", "Cone", "Cylinder", "Sphere", "Torus"]
    camera_list = ["xz-cam", "yz-cam"]
    camera_locations = [[0,-20,0], [-20,0,0]]
    camera_rotations = [[radians(90),0,0], [radians(90),0,radians(270)]]
    scale_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0]

    # For some reason blender crashes after doing this once,
    # so the loop needs to be outside this script and call Blender each time.
    sys.argv = sys.argv[sys.argv.index("--") + 1:]
    shape = shape_list[int(sys.argv[0])]
    camera = camera_list[int(sys.argv[1])]
    camera_location = camera_locations[int(sys.argv[1])]
    camera_rotation = camera_rotations[int(sys.argv[1])]
    scale = scale_list[int(sys.argv[2])]
    resolution = default_resolution*scale

    camera_location = tuple(camera_location)
    camera_rotation = tuple(camera_rotation)

    camera = camera + "-scale={}".format(int(scale*3840))

    # Delete objects in scene after opening Blender
    for o in bpy.context.scene.objects:
        # if o.type == "MESH":
        o.select_set(True)

    bpy.ops.object.delete()

    create_light()
    create_shape(shape=shape)
    so = bpy.data.objects[shape]

    shape_properties = get_shape_properties(so)

    output_dir = os.path.join(os.getcwd(), os.path.join("scale-analysis", shape))
    if not os.path.isdir(output_dir):
        os.mkdir(output_dir)
    if not os.path.isfile(os.path.join(output_dir,"shape-properties.npy")):
        np.save(os.path.join(output_dir,"shape-properties.npy"), shape_properties)

    create_camera(
        camera,
        location=camera_location,
        rotation=camera_rotation,
        focal_length=50
        )

    save_all_camera_params(output_dir, resolution, camera_location, camera_rotation)

    render_from_cam(bpy.data.objects[camera], output_dir=output_dir, resolution=resolution)

    for o in bpy.context.scene.objects:
        o.select_set(True)

    bpy.ops.object.delete()

    # update scene, if needed
    dg = bpy.context.evaluated_depsgraph_get()
    dg.update()
    end = time()
    print("Completed {}, {}, scale={}".format(shape, camera, scale))
    print("time = {} s".format(end-start))

if __name__ == "__main__":
    main()
