import numpy as np
import cv2 as cv
import pandas as pd
import math
import matplotlib.pyplot as plt
from math import radians, degrees, cos, sin, pi
from glob import glob
import sys
import os
sys.path.insert(0, os.path.join(os.getcwd(),"..\\"))
from visual_fire_power import planar_volume_reconstruction, get_flame_height, save_shape_surface_img, draw_contour

def main():
    shape_list = ["Cube", "Cone", "Cylinder", "Sphere", "Torus"]
    translation_list = [10, 5, 0, -10, -20]
    shape_volumes = [2**3, pi*1*(2/3), pi*1*2, (4/3)*pi*1, (1/8)*pi**2]
    shape_dict = dict(zip(shape_list, shape_volumes))

    shape_array = []
    translation_array = []
    volume_array = []
    height_array = []
    vol_error_array = []
    height_error_array = []

    for shape in shape_list:
        xz_file_list = glob("distance-analysis\\"+ shape + "\\xz-cam*.png")
        yz_file_list = glob("distance-analysis\\"+ shape + "\\yz-cam*.png")
        xz_K_list = glob("distance-analysis\\"+ shape + "\\xz-cam*K.npy")
        yz_K_list = glob("distance-analysis\\"+ shape + "\\yz-cam*K.npy")
        xz_RT_list = glob("distance-analysis\\"+ shape + "\\xz-cam*RT.npy")
        yz_RT_list = glob("distance-analysis\\"+ shape + "\\yz-cam*RT.npy")

        shape_properties = np.load("distance-analysis\\"+ shape + "\\shape-properties.npy")


        for i in range(len(xz_file_list)):
            img_list = [cv.imread(xz_file_list[i]), cv.imread(yz_file_list[i])]
            thresh_list = [np.greater(img[:,:,2], img[:,:,0]).astype(np.uint8)
                           * 255 for img in img_list]

            K_list = [np.load(xz_K_list[i]), np.load(yz_K_list[i])]
            RT_list = [np.load(xz_RT_list[i]), np.load(yz_RT_list[i])]
            assert len(K_list) == len(RT_list) and len(K_list) == len(img_list)

            # test_img = draw_contour(thresh_list[0])
            # cv.imshow("img", img_list[0])
            # cv.waitKey(1000)
            # cv.imshow("img",test_img)
            # cv.waitKey(1000)
            # cv.destroyAllWindows()

            volume = planar_volume_reconstruction(thresh_list, K_list, RT_list, units_scaling=1000)
            volume_error = np.round(((volume - shape_dict[shape]) /
                                     shape_dict[shape]) * 100, 3)

            flame_height = max([get_flame_height(thresh_list[j],
                                                 K_list[j],
                                                 RT_list[j])
                                for j in range(len(thresh_list))])
            flame_height_error = np.round((abs(flame_height - 2)/2) * 100, 3)

            print(shape)
            print(f"Volume = {volume} m3")
            print(f"Volume error = {volume_error}%")
            print(f"Height = {flame_height} mm")
            print(f"Height error = {flame_height_error}%")
            print(xz_file_list[i])

            filename = xz_file_list[i]
            start = filename.find("Z=") + len("Z=")
            end = filename.find(".png")
            substring = filename[start:end]
            translation = float(substring)

            surface_file = "distance-analysis\\plots\\{}\\Z={}".format(shape, substring) + "-surface.png"
            surface_img = save_shape_surface_img(thresh_list, K_list, RT_list, surface_file)

            shape_array.append(shape)
            translation_array.append(translation)
            volume_array.append(np.round(volume, 3))
            height_array.append(np.round(flame_height, 3))
            vol_error_array.append(volume_error)
            height_error_array.append(flame_height_error)

            print("Completed {} of {} files for {}".format(i+1,
                                                           len(xz_file_list),
                                                           shape))

    df = pd.DataFrame(data=np.vstack([shape_array,
                                      translation_array,
                                      volume_array,
                                      height_array,
                                      vol_error_array,
                                      height_error_array]).T,
                      columns=["shape",
                               "translation",
                               "volume_in_m3",
                               "height_in_mm",
                               "volume_error",
                               "height_error"])

    df.to_csv("blender_distance_analysis.csv")

if __name__ == "__main__":
    main()
