import numpy as np
import cv2 as cv
import pandas as pd
import math
from math import radians, degrees, cos, sin, pi
from glob import glob
import sys
import os
sys.path.insert(0, os.path.join(os.getcwd(),"..\\"))
from visual_fire_power import planar_volume_reconstruction, get_flame_height, save_shape_surface_img, draw_contour
from visual_fire_power import get_largest_contour, get_2D_flame_surface, get_contour_top_surface, get_contour_bottom_surface, uv_to_XYZ
import scipy.interpolate as interpolate


def main():
    shape_list = ["Cube", "Cone", "Cylinder", "Sphere", "Torus"]
    translation_list = [10, 5, 0, -5, -10]
    offset_list = [n*10**-3 for n in [100, 50, 0, -50, -100]]
    angle_list = [radians(a) for a in [10, 5, 0, -5, -10]]
    scale_list = [0.5, 0.75, 1, 1.5, 2]
    shape_volumes = [2**3, pi*1*(2/3), pi*1*2, (4/3)*pi*1, (1/8)*pi**2]
    vol_dict = dict(zip(shape_list, shape_volumes))
    shape_heights = [2, 2, 2, 2, 0.5]
    height_dict = dict(zip(shape_list, shape_heights))

    shape_array = []
    translation_array = []
    angle_array = []
    offset_array = []
    scale_array = []
    volume_array = []
    height_array = []
    vol_error_array = []
    height_error_array = []

    for shape in shape_list:
        xz_file_list = glob(shape + "\\xz-cam*.png")
        yz_file_list = glob(shape + "\\yz-cam*.png")
        xz_K_list = glob(shape + "\\xz-cam*K.npy")
        yz_K_list = glob(shape + "\\yz-cam*K.npy")
        xz_RT_list = glob(shape + "\\xz-cam*RT.npy")
        yz_RT_list = glob(shape + "\\yz-cam*RT.npy")

        assert len(xz_file_list) == len(yz_file_list)

        for i in range(len(xz_file_list)):
            img_list = [cv.imread(xz_file_list[i]), cv.imread(yz_file_list[i])]
            thresh_list = [np.greater(img[:,:,2], img[:,:,0]).astype(np.uint8)
                           * 255 for img in img_list]

            K_list = [np.load(xz_K_list[i]), np.load(yz_K_list[i])]
            RT_list = [np.load(xz_RT_list[i]), np.load(yz_RT_list[i])]
            assert len(K_list) == len(RT_list) and len(K_list) == len(img_list)

            # test_img = draw_contour(thresh_list[0])
            # cv.imshow("img", img_list[0])
            # cv.waitKey(1000)
            # cv.imshow("img",test_img)
            # cv.waitKey(1000)
            # cv.destroyAllWindows()
            # exit()

            start_num = len(shape)+8
            filename = "plots\\{}\\{}".format(shape, xz_file_list[i][start_num:-4]) + "-surface"
            save_shape_surface_img(thresh_list, K_list, RT_list, filename)

            volume = planar_volume_reconstruction(thresh_list, K_list, RT_list, units_scaling=1000)
            volume_error = np.round((abs(volume - vol_dict[shape]) /
                                     vol_dict[shape]) * 100, 3)


            flame_height = max([get_flame_height(thresh_list[j],
                                                 K_list[j],
                                                 RT_list[j])
                                for j in range(len(thresh_list))])
            flame_height_error = np.round((abs(flame_height - height_dict[shape]) /
                                           height_dict[shape]) * 100, 3)

            print(shape)
            print(f"Volume = {volume} m3")
            print(f"Volume error = {volume_error}%")
            print(f"Height = {flame_height} m")
            print(f"Height error = {flame_height_error}%")
            print(xz_file_list[i])

            filename = xz_file_list[i]
            start = filename.find("Z=") + len("Z=")
            end = filename.find("-angle")
            substring = filename[start:end]
            translation = float(substring)

            start = filename.find("angle=") + len("angle=")
            end = filename.find("-offset")
            substring = filename[start:end]
            angle = float(substring)

            start = filename.find("offset=") + len("offset=")
            end = filename.find("-scale")
            substring = filename[start:end]
            offset = float(substring)

            start = filename.find("scale=") + len("scale=")
            end = filename.find(".png")
            substring = filename[start:end]
            scale = float(substring)

            shape_array.append(shape)
            translation_array.append(translation)
            angle_array.append(angle)
            offset_array.append(offset)
            scale_array.append(scale)
            volume_array.append(np.round(volume, 3))
            height_array.append(np.round(flame_height, 3))
            vol_error_array.append(volume_error)
            height_error_array.append(flame_height_error)

            print("Completed {} of {} files for {}".format(i+1,
                                                           len(xz_file_list),
                                                           shape))


    df = pd.DataFrame(data=np.vstack([shape_array,
                                      translation_array,
                                      angle_array,
                                      offset_array,
                                      scale_array,
                                      volume_array,
                                      height_array,
                                      vol_error_array,
                                      height_error_array]).T,
                      columns=["shape",
                               "translation",
                               "angle",
                               "offset",
                               "scale",
                               "volume_in_m3",
                               "height_in_mm",
                               "volume_error",
                               "height_error"])

    df.to_csv("blender_sensitivity_analysis.csv")

if __name__ == "__main__":
    main()
